import click

from .processor import LogsProcessor
from .formatter import LogsFormatter
import yaml

@click.command()
@click.option('--format', '-f', type=click.Choice(['json', 'csv']), default='json')
@click.option('--logs-path', '-l', required=True)
@click.option('--out-path', '-o', required=True)
@click.option('--config-path', '-c', required=True)

def main(format, logs_path, out_path, config_path):
    config_file = open(config_path, 'r')
    config = yaml.load(config_file)
    processor = LogsProcessor(logs_path)
    formatter = LogsFormatter(out_path, format, config)
    formatter.run(processor)

if __name__ == '__main__':
    main()



import os
import gzip
import json
import re

class LogsProcessor:
    def __init__(self, logs_path):
        self.logs_path = os.path.abspath(logs_path)

    def process(self, file_path):
        file = gzip.open(file_path, 'r')

        with file as logs_file:
            for line in logs_file:
                data = self.lower_keys(json.loads(line))

                data["message"] = self.decode(data["message"])

                while 'message' in data['message']:
                    data['message'] = self.decode(data['message']['message'])

                # if 'transaction' in data['message']:
                #     data['message'] = data['message']['request']

                data["message"].pop('traceback', None)

                if 'http_request' in data["message"]:
                    if 'headers' in data["message"]["http_request"]:
                        data["message"]["http_request"]["headers"] = json.loads(data["message"]["http_request"]["headers"])
                    elif 'all' in data["message"]["http_request"]:
                        data["message"]["http_request"]['all'] = json.loads(data["message"]["http_request"]['all'])

                if 'ml_waf' in data['message']:
                    if data['message']['ml_waf']:
                        data['message']['ml_waf'] = json.loads(data['message']['ml_waf'])

                yield data['message']

    def logs(self):
        for file in os.listdir(self.logs_path):
            if file.endswith('.gz'):
                yield os.path.join(self.logs_path, file)

    def lower_keys(self, x):
        if isinstance(x, list):
            return [self.lower_keys(v) for v in x]
        elif isinstance(x, dict):
            return dict((k.lower(), self.lower_keys(v)) for k, v in x.items())
        else:
            return x

    def decode(self, encoded_string):
        try:
            encoded_string = json.loads(encoded_string)
        except json.decoder.JSONDecodeError:
            encoded_string = re.sub(r'\\x([a-fA-F0-9]{2})', r'\\u00\1', encoded_string)
            encoded_string = json.loads(encoded_string)

        return self.lower_keys(encoded_string)



import ntpath
import re
import os
import json
import csv
import sys

class LogsFormatter:
    def __init__(self, output_path, format, config):
        self.output_path = output_path
        self.format = format
        self.config = config

    def run(self, logs_processor):
        matcher = re.compile("_[0-9]{8}-[0-9]{4,5}[PM|AM]+-(.*).log.gz")

        for log_file in logs_processor.logs():
            file_name = ntpath.basename(log_file)

            try:
                logs_type = matcher.search(file_name).groups()[0]
            except AttributeError:
                print('ERROR: when loading ' + file_name)
                continue


            file_name = os.path.splitext(file_name)[0]
            output = os.path.join(self.output_path, file_name + '.' + self.format)

            if os.path.exists(output):
                print('INFO: skipping ' + output + ' already processed')
                continue

            if self.format == 'json':
                self.process_json(logs_processor.process(log_file), output)

            if self.format == 'csv':
                self.process_csv(logs_processor.process(log_file), output, logs_type)

    def process_json(self, logs, output_path):
        print('INFO: writing ' + output_path)
        output = open(output_path, 'w')
        for log in logs:
            output.write(json.dumps(log) + '\n')
        output.close

    def process_csv(self, logs, output_path, logs_type):
        try:
            output_keys = self.config['output'][logs_type]
        except:
            print('ERROR: missing output definition for "' + logs_type + '" possible keys: ')
            print(next(logs).keys())
            sys.exit(1)

        print('INFO: writing ' + output_path)
        output = open(output_path, 'w')
        writer = csv.DictWriter(output, fieldnames=output_keys)
        writer.writeheader()
        for log in logs:
            filtered_log = { key:value for key, value in log.items() if key in output_keys }
            writer.writerow(filtered_log)



from setuptools import setup

setup(
    name = 'logscli',
    version = '0.1.0',
    author = 'Karolis Mazukna',
    author_email = 'karolis@zenedge.com',
    url = 'https://github.com/zenedge/logs_cli',
    packages = ['logscli'],
    entry_points = {
        'console_scripts': [
            'logscli = logscli.__main__:main'
        ]
    },
    install_requires = {
      'pyaml',
      'click'
    }
)
